#!/usr/bin/env python
"""Run all tests"""

# Modified from test.py in UFL

__author__ = "Martin Alnes (martinal@simula.no) and Anders Logg (logg@simula.no)"
__date__ = "2008-03-12 -- 2011-04-08"

import glob, os, sys, unittest

def discover_packages():
    # Looking at tests in all test_foo/ packages
    packages = sorted(f for f in glob.glob("test_*")
                      if os.path.exists(os.path.join(f,'__init__.py')))
    return packages

def discover_tests(p):
    import glob, os
    # Running tests from all test_foo.py files in each package
    tests = sorted(os.path.basename(f.replace(".py", ""))
                   for f in glob.glob(os.path.join(p,"test_*.py")))
    return tests

def run_suite(package, tests):
    assert tests
    loader = unittest.TestLoader()
    p = __import__(package, fromlist=tests)
    modules = [getattr(p,test) for test in tests]
    suite = loader.loadTestsFromModule(modules[0])
    for m in modules[1:]:
        suite.addTests(loader.loadTestsFromModule(m))
    runner = unittest.TextTestRunner(verbosity=2)
    return runner.run(suite)

def main(args):
    s, f = 0, 0
    cwd = os.path.abspath(os.curdir)

    tfilter = []
    for a in args:
        if a.startswith('test_'):
            tfilter.append(a)
        else:
            print "Ignoring argument", a
    if tfilter:
        print "Running only tests", tfilter

    packages = discover_packages()
    for p in packages:
        tests = discover_tests(p)
        if tfilter:
            # Use only tests found in args
            tests = [t for t in tests if t in tfilter]
            if not tests:
                continue
        result = run_suite(p, tests)
        os.chdir(cwd) # In case tests mess up current directory...
        if result.wasSuccessful():
            s += 1
        else:
            f += 1

    if not f:
        print "All tests finished successfully."
        return 0
    else:
        print "Not all tests finished successfully."
        return 1

if __name__ == "__main__":
    sys.exit(main(sys.argv[1:]))
