#!/usr/bin/env python

__authors__ = "Martin Sandve Alnes"
__date__ = "2012-05-16"

from unittest import TestCase
import swiginac
import SyFi

class TestGeometry(TestCase):

    def _validate_vertex(self, v, dim):
        if len(v) != dim:
            print
            print v
            print
        self.assertEqual(len(v), dim)

    def _validate_line(self, cell, dim):
        self.assertEqual(cell.no_space_dim(), 1)
        #self.assertEqual(cell.no_space_dim(), dim)

        self.assertEqual(cell.no_vertices(), 2)
        for i in range(2):
            self._validate_vertex(cell.vertex(i), dim)

    def _validate_triangle(self, cell, dim):
        self.assertEqual(cell.no_space_dim(), 2)
        #self.assertEqual(cell.no_space_dim(), dim)

        self.assertEqual(cell.no_vertices(), 3)
        for i in range(3):
            self._validate_vertex(cell.vertex(i), dim)
        for i in range(3):
            self._validate_line(cell.line(i), dim)

    def _validate_tetrahedron(self, cell, dim):
        self.assertEqual(cell.no_space_dim(), 3)
        #self.assertEqual(cell.no_space_dim(), dim)

        self.assertEqual(cell.no_vertices(), 4)
        for i in range(4):
            self._validate_vertex(cell.vertex(i), dim)
        for i in range(6):
            self._validate_line(cell.line(i), dim)
        for i in range(4):
            self._validate_triangle(cell.triangle(i), dim)

    def _validate_rectangle(self, cell, dim):
        self.assertEqual(cell.no_space_dim(), 2)
        #self.assertEqual(cell.no_space_dim(), dim)

        self.assertEqual(cell.no_vertices(), 4)
        for i in range(4):
            self._validate_vertex(cell.vertex(i), dim)
        for i in range(4):
            self._validate_line(cell.line(i), dim)

    def _validate_box(self, cell, dim):
        self.assertEqual(cell.no_space_dim(), 3)
        #self.assertEqual(cell.no_space_dim(), dim)

        self.assertEqual(cell.no_vertices(), 8)
        for i in range(6):
            self._validate_vertex(cell.vertex(i), dim)
        for i in range(12):
            self._validate_line(cell.line(i), dim)
        for i in range(6):
            self._validate_rectangle(cell.rectangle(i), dim)

    def test_reference_line_dimensions(self):
        d = 1
        SyFi.initSyFi(d)
        c = SyFi.ReferenceLine()
        self._validate_line(c, d)

    def test_reference_triangle_dimensions(self):
        d = 2
        SyFi.initSyFi(d)
        c = SyFi.ReferenceTriangle()
        self._validate_triangle(c, d)

    def test_reference_rectangle_dimensions(self):
        d = 2
        SyFi.initSyFi(d)
        c = SyFi.ReferenceRectangle()
        self._validate_rectangle(c, d)

    def test_reference_tetrahedron_dimensions(self):
        d = 3
        SyFi.initSyFi(d)
        c = SyFi.ReferenceTetrahedron()
        self._validate_tetrahedron(c, d)

    def test_reference_box_dimensions(self):
        d = 3
        SyFi.initSyFi(d)
        c = SyFi.ReferenceBox()
        self._validate_box(c, d)

    def test_spacetime_dimensions(self):
        l = SyFi.ReferenceLine()
        t = SyFi.ReferenceTriangle()
        sd = SyFi.SpaceTimeDomain(l, t)
        #self.assertEqual(TODO)
